% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/character.r
\name{cat0}
\alias{cat0}
\title{Concatenate and Print with No Separator}
\usage{
cat0(..., file = "", fill = FALSE, labels = NULL, append = FALSE)
}
\arguments{
\item{...}{\strong{R} objects (see \link{cat} for the types of objects allowed).}

\item{file}{a connection, or a character string naming the file to
print to. If "" (the default), \code{cat0} prints to the standard output
connection, the console unless redirected by sink.}

\item{fill}{a logical or (positive) numeric controlling how the
output is broken into successive lines. If \code{FALSE} (default), only
newlines created explicitly by \code{"\\n"} are printed. Otherwise, the
output is broken into lines with print width equal to the option
width if fill is \code{TRUE}, or the value of fill if this is numeric.
Linefeeds are only inserted between elements, strings wider than
fill are not wrapped. Non-positive fill values are ignored, with a
warning.}

\item{labels}{character vector of labels for the lines printed.
Ignored if fill is \code{FALSE}.}

\item{append}{logical. Only used if the argument file is the name
of file (and not a connection or \code{"|cmd"}). If \code{TRUE} output will be
appended to file; otherwise, it will overwrite the contents of file.}
}
\value{
None (invisible \code{NULL}).
}
\description{
Outputs the objects, concatenating the representations
with no separator.
}
\details{
All arguments are passed to a call to \link{cat} with \code{sep} set to \code{""}.
}
\examples{
cat(letters[1:3])
cat0(letters[1:3])

cat(
  paste(letters, 100 * 1:26),
  fill = TRUE, labels = paste0("{", 1:10, "}:")
)
cat0(
  paste(letters, 100 * 1:26),
  fill = TRUE, labels = paste0("{", 1:10, "}:")
)
}
\seealso{
\link{cat}, \link{print}, \link{format} and \link{paste} which concatenates into
a string.
}
