% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suppr-package.R
\docType{package}
\name{suppr-package}
\alias{suppr}
\alias{suppr-package}
\title{suppr: Supplementary Idiomatic Utilities and Extensions}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Miscellaneous supplementary functions designed to follow idiomatic 'R' conventions. Some functions are simple wrappers that reduce repetitive code, while others address common tasks or extend existing 'R' functions.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://lj-jenkins.github.io/suppr/}
  \item \url{https://github.com/LJ-Jenkins/suppr}
  \item Report bugs at \url{https://github.com/LJ-Jenkins/suppr/issues}
}

}
\author{
\strong{Maintainer}: Luke Jenkins \email{luke-jenkins-dev@outlook.com} (\href{https://orcid.org/0000-0002-7206-7242}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Luke Jenkins \email{luke-jenkins-dev@outlook.com} (\href{https://orcid.org/0000-0002-7206-7242}{ORCID}) [copyright holder]
}

}
\keyword{internal}
