% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/character.r
\name{bckQuote}
\alias{bckQuote}
\title{Backquote Text}
\usage{
bckQuote(x)
}
\arguments{
\item{x}{an \strong{R} object, to be coerced to a \link{character} vector.}
}
\value{
A character vector of the same length as \code{x} (after any coercion).
}
\description{
Backquote text by combining with backticks.
}
\examples{
bckQuote("example")
bckQuote(c("one", "two", "three"))
bckQuote(123)
}
\seealso{
\link{Quotes}, \link{dQuote} and \link{sQuote}.
}
