% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.r
\name{empty.list}
\alias{empty.list}
\title{Create an Empty list of Given Length}
\usage{
empty.list(length = 0L)
}
\arguments{
\item{length}{integer, specified length of the output list.}
}
\value{
an empty list of given length.
}
\description{
For a given length, create an empty list of that length.
}
\details{
This is a simple wrapper of \code{vector("list", length)} with a
clearer naming convention.
}
\examples{
empty.list()
empty.list(5L)
empty.list(length = 3L)
}
\seealso{
\link{vector}, \link{na.vector}
}
